@@ -1,4 +1,4 @@
-use std::{mem, ptr};
+use std::{ffi::c_void, mem, ptr};
 
 use dlopen::wrapper::WrapperApi;
 
@@ -17,7 +17,7 @@ pub enum hostfxr_delegate_type {
 
 pub type hostfxr_error_writer_fn = unsafe extern "C" fn(message: *const char_t);
 
-pub type hostfxr_handle = *const ();
+pub type hostfxr_handle = *const c_void;
 
 /// A structure that stores parameters which are common to all forms of initialization.
 #[repr(C)]
@@ -43,6 +43,7 @@ pub struct hostfxr_initialize_parameters {
 impl hostfxr_initialize_parameters {
     /// Creates a new instance of [`hostfxr_initialize_parameters`] with the given `host_path`.
     /// The `size` field is set accordingly to the size of the struct and `dotnet_root` to [`ptr::null()`].
+    #[must_use]
     pub fn with_host_path(host_path: *const char_t) -> hostfxr_initialize_parameters {
         hostfxr_initialize_parameters {
             size: mem::size_of::<hostfxr_initialize_parameters>(),
@@ -52,6 +53,7 @@ impl hostfxr_initialize_parameters {
     }
     /// Creates a new instance of [`hostfxr_initialize_parameters`] with the given `dotnet_root`.
     /// The `size` field is set accordingly to the size of the struct and `host_path` to [`ptr::null()`].
+    #[must_use]
     pub fn with_dotnet_root(dotnet_root: *const char_t) -> hostfxr_initialize_parameters {
         hostfxr_initialize_parameters {
             size: mem::size_of::<hostfxr_initialize_parameters>(),
@@ -158,8 +160,8 @@ pub struct HostfxrLib {
     ///
     /// This function does not load the runtime.
     ///
-    /// [`Success`]: crate::hostfxr::KnownHostExitCode::Success
-    /// [`HostInvalidState`]: crate::hostfxr::KnownHostExitCode::HostInvalidState
+    /// [`Success`]: crate::bindings::StatusCode::Success
+    /// [`HostInvalidState`]: crate::bindings::StatusCode::HostInvalidState
     hostfxr_initialize_for_dotnet_command_line: unsafe extern "C" fn(
         argc: i32,
         argv: *const *const char_t,
@@ -201,10 +203,10 @@ pub struct HostfxrLib {
     /// initializations. In the case of [`Success_DifferentRuntimeProperties`], it is left to the consumer to verify that
     /// the difference in properties is acceptable.
     ///
-    /// [`Success`]: crate::hostfxr::KnownHostExitCode::Success
-    /// [`Success_HostAlreadyInitialized`]: crate::hostfxr::KnownHostExitCode::Success_HostAlreadyInitialized
-    /// [`Success_DifferentRuntimeProperties`]: crate::hostfxr::KnownHostExitCode::Success_DifferentRuntimeProperties
-    /// [`CoreHostIncompatibleConfig`]: crate::hostfxr::KnownHostExitCode::CoreHostIncompatibleConfig
+    /// [`Success`]: crate::bindings::StatusCode::Success
+    /// [`Success_HostAlreadyInitialized`]: crate::bindings::StatusCode::Success_HostAlreadyInitialized
+    /// [`Success_DifferentRuntimeProperties`]: crate::bindings::StatusCode::Success_DifferentRuntimeProperties
+    /// [`CoreHostIncompatibleConfig`]: crate::bindings::StatusCode::CoreHostIncompatibleConfig
     hostfxr_initialize_for_runtime_config: unsafe extern "C" fn(
         runtime_config_path: *const char_t,
         parameters: *const hostfxr_initialize_parameters,
@@ -384,13 +386,13 @@ pub struct HostfxrLib {
 /// [`hdt_load_assembly_and_get_function_pointer`]: hostfxr_delegate_type::`hdt_load_assembly_and_get_function_pointer
 /// [`UNMANAGED_CALLERS_ONLY_METHOD`]: crate::bindings::consts::UNMANAGED_CALLERS_ONLY_METHOD
 /// [`UnmanagedCallersOnlyAttribute`]: https://docs.microsoft.com/en-us/dotnet/api/system.runtime.interopservices.unmanagedcallersonlyattribute
-pub type load_assembly_and_get_function_pointer_fn = unsafe extern "C" fn(
+pub type load_assembly_and_get_function_pointer_fn = unsafe extern "system" fn(
     assembly_path: *const char_t,
     type_name: *const char_t,
     method_name: *const char_t,
     delegate_type_name: *const char_t,
-    reserved: *const (),
-    /*out*/ delegate: *mut *const (),
+    reserved: *const c_void,
+    /*out*/ delegate: *mut *const c_void,
 ) -> i32;
 
 /// Signature of delegate returned by [`hostfxr_get_runtime_delegate`] for type [`hdt_get_function_pointer`]
@@ -413,14 +415,14 @@ pub type load_assembly_and_get_function_pointer_fn = unsafe extern "C" fn(
 /// [`hostfxr_get_runtime_delegate`]: struct.HostfxrLib.html#method.hostfxr_get_runtime_delegate
 /// [`UNMANAGED_CALLERS_ONLY_METHOD`]: crate::bindings::consts::UNMANAGED_CALLERS_ONLY_METHOD
 /// [`UnmanagedCallersOnlyAttribute`]: https://docs.microsoft.com/en-us/dotnet/api/system.runtime.interopservices.unmanagedcallersonlyattribute
-pub type get_function_pointer_fn = unsafe extern "C" fn(
+pub type get_function_pointer_fn = unsafe extern "system" fn(
     type_name: *const char_t,
     method_name: *const char_t,
     delegate_type_name: *const char_t,
-    load_context: *const (),
-    reserved: *const (),
-    /*out*/ delegate: *mut *const (),
+    load_context: *const c_void,
+    reserved: *const c_void,
+    /*out*/ delegate: *mut *const c_void,
 ) -> i32;
 
 /// Signature of delegate returned by [`load_assembly_and_get_function_pointer_fn`] when `delegate_type_name == null` (default)
-pub type component_entry_point_fn = unsafe extern "C" fn(*const (), size_t) -> i32;
+pub type component_entry_point_fn = unsafe extern "system" fn(*const c_void, size_t) -> i32;
