@@ -10,7 +10,7 @@ use super::type_aliases::{char_t, size_t};
     link(name = "stdc++", kind = "dylib")
 )]
 #[cfg_attr(target_os = "macos", link(name = "c++", kind = "dylib"))]
-extern "C" {
+extern "system" {
     /// Get the path to the hostfxr library
     ///
     /// # Arguments
@@ -28,7 +28,7 @@ extern "C" {
     ///
     /// # Return value
     ///  * 0 on success, otherwise failure
-    ///  * 0x80008098 - `buffer` is too small ([`HostApiBufferTooSmall`](crate::hostfxr::KnownHostExitCode::HostApiBufferTooSmall))
+    ///  * 0x80008098 - `buffer` is too small ([`HostApiBufferTooSmall`](crate::error::HostingError::HostApiBufferTooSmall))
     ///
     /// # Remarks
     /// The full search for the hostfxr library is done on every call. To minimize the need
@@ -58,6 +58,7 @@ pub struct get_hostfxr_parameters {
 impl get_hostfxr_parameters {
     /// Creates a new instance of [`get_hostfxr_parameters`] with the given `dotnet_root`.
     /// The `size` field is set accordingly to the size of the struct and `assembly_path` to [`ptr::null()`].
+    #[must_use]
     pub fn with_dotnet_root(dotnet_root: *const char_t) -> get_hostfxr_parameters {
         get_hostfxr_parameters {
             size: size_of::<get_hostfxr_parameters>(),
@@ -67,6 +68,7 @@ impl get_hostfxr_parameters {
     }
     /// Creates a new instance of [`get_hostfxr_parameters`] with the given `assembly_path`.
     /// The `size` field is set accordingly to the size of the struct and `dotnet_root` to [`ptr::null()`].
+    #[must_use]
     pub fn with_assembly_path(assembly_path: *const char_t) -> get_hostfxr_parameters {
         get_hostfxr_parameters {
             size: size_of::<get_hostfxr_parameters>(),
