@@ -7,26 +7,29 @@ use crate::{
             load_assembly_and_get_function_pointer_fn,
         },
     },
+    error::{HostingError, HostingResult, HostingSuccess},
     pdcstring::PdCStr,
-    Error,
 };
-
+use num_enum::TryFromPrimitive;
 use std::{
+    convert::TryFrom,
     mem::{self, MaybeUninit},
+    path::PathBuf,
     ptr,
 };
-
-use super::HostExitCode;
+use thiserror::Error;
 
 /// A function pointer for a method with the default signature.
 pub type MethodWithDefaultSignature = component_entry_point_fn;
+/// A opaque type representing some method with an unknown signature.
+pub enum SomeMethod {}
 /// A function pointer for a method with an unknown signature.
-pub type MethodWithUnknownSignature = *const ();
+pub type MethodWithUnknownSignature = *const SomeMethod;
 
 /// A struct for loading pointers to managed functions for a given [`HostfxrContext`].
 ///
 /// [`HostfxrContext`]: super::HostfxrContext
-#[derive(Copy, Clone, PartialEq, Eq, Debug, Hash)]
+#[derive(Copy, Clone)]
 pub struct DelegateLoader {
     pub(crate) get_load_assembly_and_get_function_pointer:
         load_assembly_and_get_function_pointer_fn,
@@ -40,20 +43,32 @@ impl DelegateLoader {
         type_name: *const char_t,
         method_name: *const char_t,
         delegate_type_name: *const char_t,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
         let mut delegate = MaybeUninit::uninit();
 
-        let result = (self.get_load_assembly_and_get_function_pointer)(
-            assembly_path,
-            type_name,
-            method_name,
-            delegate_type_name,
-            ptr::null(),
-            delegate.as_mut_ptr(),
-        );
-        HostExitCode::from(result).to_result()?;
+        let result = unsafe {
+            (self.get_load_assembly_and_get_function_pointer)(
+                assembly_path,
+                type_name,
+                method_name,
+                delegate_type_name,
+                ptr::null(),
+                delegate.as_mut_ptr(),
+            )
+        };
+        GetFunctionPointerError::from_status_code(result)?;
 
-        Ok(delegate.assume_init())
+        Ok(unsafe { delegate.assume_init() }.cast())
+    }
+
+    fn _validate_assembly_path(
+        assembly_path: impl AsRef<PdCStr>,
+    ) -> Result<(), GetFunctionPointerError> {
+        if PathBuf::from(assembly_path.as_ref().to_os_string()).exists() {
+            Ok(())
+        } else {
+            Err(GetFunctionPointerError::AssemblyNotFound)
+        }
     }
 
     unsafe fn _get_function_pointer(
@@ -61,20 +76,22 @@ impl DelegateLoader {
         type_name: *const char_t,
         method_name: *const char_t,
         delegate_type_name: *const char_t,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
         let mut delegate = MaybeUninit::uninit();
 
-        let result = (self.get_function_pointer)(
-            type_name,
-            method_name,
-            delegate_type_name,
-            ptr::null(),
-            ptr::null(),
-            delegate.as_mut_ptr(),
-        );
-        HostExitCode::from(result).to_result()?;
+        let result = unsafe {
+            (self.get_function_pointer)(
+                type_name,
+                method_name,
+                delegate_type_name,
+                ptr::null(),
+                ptr::null(),
+                delegate.as_mut_ptr(),
+            )
+        };
+        GetFunctionPointerError::from_status_code(result)?;
 
-        Ok(delegate.assume_init())
+        Ok(unsafe { delegate.assume_init() }.cast())
     }
 
     /// Calling this function will load the specified assembly in isolation (into its own `AssemblyLoadContext`)
@@ -99,7 +116,8 @@ impl DelegateLoader {
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
         delegate_type_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
+        Self::_validate_assembly_path(assembly_path.as_ref())?;
         unsafe {
             self._load_assembly_and_get_function_pointer(
                 assembly_path.as_ref().as_ptr(),
@@ -130,7 +148,8 @@ impl DelegateLoader {
         assembly_path: impl AsRef<PdCStr>,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithDefaultSignature, Error> {
+    ) -> Result<MethodWithDefaultSignature, GetFunctionPointerError> {
+        Self::_validate_assembly_path(assembly_path.as_ref())?;
         unsafe {
             self._load_assembly_and_get_function_pointer(
                 assembly_path.as_ref().as_ptr(),
@@ -164,7 +183,8 @@ impl DelegateLoader {
         assembly_path: impl AsRef<PdCStr>,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
+        Self::_validate_assembly_path(assembly_path.as_ref())?;
         unsafe {
             self._load_assembly_and_get_function_pointer(
                 assembly_path.as_ref().as_ptr(),
@@ -190,7 +210,7 @@ impl DelegateLoader {
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
         delegate_type_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
         unsafe {
             self._get_function_pointer(
                 type_name.as_ref().as_ptr(),
@@ -213,14 +233,14 @@ impl DelegateLoader {
         &self,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithDefaultSignature, Error> {
+    ) -> Result<MethodWithDefaultSignature, GetFunctionPointerError> {
         unsafe {
-            let fn_ptr = self._get_function_pointer(
+            self._get_function_pointer(
                 type_name.as_ref().as_ptr(),
                 method_name.as_ref().as_ptr(),
                 ptr::null(),
-            );
-            mem::transmute(fn_ptr)
+            )
+            .map(|fn_ptr| mem::transmute(fn_ptr))
         }
     }
 
@@ -239,7 +259,7 @@ impl DelegateLoader {
         &self,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
         unsafe {
             self._get_function_pointer(
                 type_name.as_ref().as_ptr(),
@@ -254,10 +274,9 @@ impl DelegateLoader {
 /// assembly from the given path on the first access.
 ///
 /// [`HostfxrContext`]: super::HostfxrContext
-#[derive(Copy, Clone, PartialEq, Eq, Debug, Hash)]
 pub struct AssemblyDelegateLoader<A: AsRef<PdCStr>> {
     loader: DelegateLoader,
-    assembly_path: Option<A>,
+    assembly_path: A,
 }
 
 impl<A: AsRef<PdCStr>> AssemblyDelegateLoader<A> {
@@ -266,7 +285,7 @@ impl<A: AsRef<PdCStr>> AssemblyDelegateLoader<A> {
     pub fn new(loader: DelegateLoader, assembly_path: A) -> Self {
         Self {
             loader,
-            assembly_path: Some(assembly_path),
+            assembly_path,
         }
     }
 
@@ -284,22 +303,17 @@ impl<A: AsRef<PdCStr>> AssemblyDelegateLoader<A> {
     ///  * `delegate_type_name`:
     ///     Assembly qualified delegate type name for the method signature.
     pub fn get_function_pointer(
-        &mut self,
+        &self,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
         delegate_type_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
-        if let Some(assembly_path) = self.assembly_path.take() {
-            self.loader.load_assembly_and_get_function_pointer(
-                assembly_path,
-                type_name,
-                method_name,
-                delegate_type_name,
-            )
-        } else {
-            self.loader
-                .get_function_pointer(type_name, method_name, delegate_type_name)
-        }
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
+        self.loader.load_assembly_and_get_function_pointer(
+            self.assembly_path.as_ref(),
+            type_name,
+            method_name,
+            delegate_type_name,
+        )
     }
 
     /// If this is the first loaded function pointer, calling this function will load the specified assembly in
@@ -315,21 +329,16 @@ impl<A: AsRef<PdCStr>> AssemblyDelegateLoader<A> {
     ///     Name of the method on the `type_name` to find. The method must be static and must match the following signature:
     ///     `public delegate int ComponentEntryPoint(IntPtr args, int sizeBytes);`
     pub fn get_function_pointer_with_default_signature(
-        &mut self,
+        &self,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithDefaultSignature, Error> {
-        if let Some(assembly_path) = self.assembly_path.take() {
-            self.loader
-                .load_assembly_and_get_function_pointer_with_default_signature(
-                    assembly_path,
-                    type_name,
-                    method_name,
-                )
-        } else {
-            self.loader
-                .get_function_pointer_with_default_signature(type_name, method_name)
-        }
+    ) -> Result<MethodWithDefaultSignature, GetFunctionPointerError> {
+        self.loader
+            .load_assembly_and_get_function_pointer_with_default_signature(
+                self.assembly_path.as_ref(),
+                type_name,
+                method_name,
+            )
     }
 
     /// If this is the first loaded function pointer, calling this function will load the specified assembly in
@@ -347,20 +356,80 @@ impl<A: AsRef<PdCStr>> AssemblyDelegateLoader<A> {
     /// [`UnmanagedCallersOnlyAttribute`]: https://docs.microsoft.com/en-us/dotnet/api/system.runtime.interopservices.unmanagedcallersonlyattribute
     /// [`\[UnmanagedCallersOnly\]`]: https://docs.microsoft.com/en-us/dotnet/api/system.runtime.interopservices.unmanagedcallersonlyattribute
     pub fn get_function_pointer_for_unmanaged_callers_only_method(
-        &mut self,
+        &self,
         type_name: impl AsRef<PdCStr>,
         method_name: impl AsRef<PdCStr>,
-    ) -> Result<MethodWithUnknownSignature, Error> {
-        if let Some(assembly_path) = self.assembly_path.take() {
-            self.loader
-                .load_assembly_and_get_function_pointer_for_unmanaged_callers_only_method(
-                    assembly_path,
-                    type_name,
-                    method_name,
-                )
-        } else {
-            self.loader
-                .get_function_pointer_for_unmanaged_callers_only_method(type_name, method_name)
-        }
+    ) -> Result<MethodWithUnknownSignature, GetFunctionPointerError> {
+        self.loader
+            .load_assembly_and_get_function_pointer_for_unmanaged_callers_only_method(
+                self.assembly_path.as_ref(),
+                type_name,
+                method_name,
+            )
     }
 }
+
+/// Enum for errors that can occur while loading a managed assembly or managed function pointers.
+#[derive(Error, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub enum GetFunctionPointerError {
+    /// An error occured inside the hosting components.
+    #[error("Error from hosting components: {}.", .0)]
+    Hosting(#[from] HostingError),
+
+    /// A type with the specified name could not be found or loaded.
+    #[error("Failed to load type containing method of delegate type.")]
+    TypeNotFound,
+
+    /// A method with the required signature and name could not be found.
+    #[error("Specified method does not exists or has an incompatible signature.")]
+    MissingMethod,
+
+    /// The specified assembly could not be found.
+    #[error("The specified assembly could not be found.")]
+    AssemblyNotFound,
+
+    /// The target method is not annotated with [`UnmanagedCallersOnly`](https://docs.microsoft.com/en-us/dotnet/api/system.runtime.interopservices.unmanagedcallersonlyattribute).
+    #[error("The target method is not annotated with UnmanagedCallersOnly.")]
+    MethodNotUnmanagedCallersOnly,
+
+    /// Some other unknown error occured.
+    #[error("Unknown error code: {}", format!("{:#08X}", .0))]
+    Other(u32),
+}
+
+impl GetFunctionPointerError {
+    #[allow(clippy::cast_sign_loss)]
+    pub fn from_status_code(code: i32) -> Result<HostingSuccess, Self> {
+        let code = code as u32;
+        match HostingResult::known_from_status_code(code) {
+            Ok(HostingResult(Ok(code))) => return Ok(code),
+            Ok(HostingResult(Err(code))) => return Err(GetFunctionPointerError::Hosting(code)),
+            _ => {}
+        }
+        match HResult::try_from(code) {
+            Ok(HResult::COR_E_TYPELOAD) => return Err(Self::TypeNotFound),
+            Ok(HResult::COR_E_MISSINGMETHOD | HResult::COR_E_ARGUMENT) => {
+                return Err(Self::MissingMethod)
+            }
+            Ok(HResult::FILE_NOT_FOUND) => return Err(Self::AssemblyNotFound),
+            Ok(HResult::COR_E_INVALIDOPERATION) => return Err(Self::MethodNotUnmanagedCallersOnly),
+            _ => {}
+        }
+        Err(Self::Other(code))
+    }
+}
+
+#[repr(u32)]
+#[non_exhaustive]
+#[derive(TryFromPrimitive, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[allow(non_camel_case_types)]
+#[rustfmt::skip]
+enum HResult {
+    E_POINTER = 0x8000_4003,                 // System.ArgumentNullException
+    COR_E_ARGUMENTOUTOFRANGE = 0x8013_1502,  // System.ArgumentOutOfRangeException (reserved was not 0)
+    COR_E_TYPELOAD = 0x8013_1522,            // invalid type
+    COR_E_MISSINGMETHOD = 0x8013_1513,       // invalid method
+    /*COR_E_*/FILE_NOT_FOUND = 0x8007_0002,  // assembly with specified name not found (from type name)
+    COR_E_ARGUMENT = 0x8007_0057,            // invalid method signature or method not found
+    COR_E_INVALIDOPERATION = 0x8013_1509,    // invalid assembly path or not unmanaged,
+}
